/*
 * @(#)PropertyExpander.java	1.5 00/01/14
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits reserves.
 */

package com.sun.security.auth.login;

import java.security.GeneralSecurityException;

/**
 * A utility class to expand properties embedded in a string.
 * Strings of the form ${some.property.name} are expanded to
 * be the value of the property. Also, the special ${/} property
 * is expanded to be the same as file.separator. If a property
 * is not set, a GeneralSecurityException will be thrown.
 *
 * @version 1.5, 01/14/00
 * @author Roland Schemers
 */
class PropertyExpander {

    static class ExpandException extends GeneralSecurityException {
	public ExpandException(String msg) {
	    super(msg);
	}
    }

    public static String expand(String value)
	throws ExpandException
    {
	if (value == null)
	    return null;

	int p = value.indexOf("${", 0);

	// no special characters
	if (p == -1) return value;

	StringBuffer sb = new StringBuffer(value.length());
	int max = value.length();
	int i = 0;  // index of last character we copied

    scanner:
	while (p < max) {
	    if (p > i) {
		// copy in anything before the special stuff
		sb.append(value.substring(i, p));
		i = p;
	    }
	    int pe = p+2;
	    while ((pe < max) && (value.charAt(pe) != '}')) {
		pe++;
	    }
	    if (pe == max) {
		// no matching '}' found, just add in as normal text
		sb.append(value.substring(p, pe));
		break scanner;
	    }
	    String prop = value.substring(p+2, pe);
	    //System.out.println("expanding ("+prop+")");
	    if (prop.equals("/")) {
		sb.append(java.io.File.separatorChar);
	    } else {
		String val = System.getProperty(prop);
		if (val != null) {
		    sb.append(val);
		} else {
		    java.text.MessageFormat form = new java.text.MessageFormat
			(sun.security.util.ResourcesMgr.getString
				("unable to expand property prop",
				"com.sun.security.auth.Resources"));
		    Object[] source = {prop};
		    throw new ExpandException(form.format(source));
		}
	    }
	    i = pe+1;
	    p = value.indexOf("${", i);
	    if (p == -1) {
		// no more to expand. copy in any extra
		if (i < max) {
		    sb.append(value.substring(i, max));
		} 
		// break out of loop
		break scanner;
	    }
	}
	return sb.toString();
    }

    public static void main(String args[]) throws Exception
    {
	System.out.println(expand(args[0]));
    }
}
