/*
 * @(#)DropTargetAdapter.java	1.2 01/01/05
 *
 * Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package java.awt.dnd;

/**
 * An abstract adapter class for receiving drop target events. The methods in
 * this class are empty. This class exists only as a convenience for creating
 * listener objects.
 * <p>
 * Extend this class to create a <code>DropTargetEvent</code> listener 
 * and override the methods for the events of interest. (If you implement the 
 * <code>DropTargetListener</code> interface, you have to define all of
 * the methods in it. This abstract class defines a null implementation for
 * every method except <code>drop(DropTargetDropEvent)</code>, so you only have
 * to define methods for events you care about.) You must provide an
 * implementation for at least <code>drop(DropTargetDropEvent)</code>. This
 * method cannot have a null implementation because its specification requires
 * that you either accept or reject the drop, and, if accepted, indicate
 * whether the drop was successful.
 * <p>
 * Create a listener object using the extended class and then register it with 
 * a <code>DropTarget</code>. When the drag enters, moves over, or exits
 * a drop site, when the drop action changes, and when the drop occurs, the
 * relevant method in the listener object is invoked, and the
 * <code>DropTargetEvent</code> is passed to it.
 *
 * @see DropTargetEvent
 * @see DropTargetListener
 *
 * @author David Mendenhall
 * @version 1.2, 01/05/01
 * @since 1.4
 */
public abstract class DropTargetAdapter implements DropTargetListener {

    /**
     * Called when a drag operation has 
     * encountered the <code>DropTarget</code>.
     *
     * @param dtde the <code>DropTargetDragEvent</code> 
     */
    public void dragEnter(DropTargetDragEvent dtde) {}

    /**
     * Called when a drag operation is ongoing 
     * on the <code>DropTarget</code>.
     *
     * @param dtde the <code>DropTargetDragEvent</code> 
     */
    public void dragOver(DropTargetDragEvent dtde) {}

    /**
     * Called if the user has modified 
     * the current drop gesture.
     *
     * @param dtde the <code>DropTargetDragEvent</code>
     */
    public void dropActionChanged(DropTargetDragEvent dtde) {}

    /**
     * The drag operation has departed 
     * the <code>DropTarget</code> without dropping.
     *
     * @param dte the <code>DropTargetEvent</code> 
     */
    public void dragExit(DropTargetEvent dte) {}
}
