/*
 * @(#)AssertionError.java	1.2 00/05/24
 *
 * Copyright 1994-2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package java.lang;

/**
 * Thrown to indicate that an assertion has failed.
 *
 * <p>The seven one-argument public constructors provided by this
 * class ensure that the assertion error returned by the invocation:
 * <pre>
 *     new AssertionError(<i>expression</i>)
 * </pre>
 * has as its detail message the <i>string conversion</i> of <i>expression</i>
 * (as defined in <i>The Java Language Specification</i>, Section 15.17.1.1),
 * regardless of the type of <i>expression</i>.
 *
 * @version 1.2, 05/24/00
 * @since   JDK1.4
 */
public class AssertionError extends Error {
    /**
     * Constructs an AssertionError with no detail message.
     */
    public AssertionError() {
    }

    /**
     * This internal constructor does no processing on its string argument,
     * even if it is a null reference.  The public constructors will
     * never call this constructor with a null argument.
     */
    private AssertionError(String detailMessage) {
        super(detailMessage);
    }

    /**
     * Constructs an AssertionError with its detail message derived from
     * the specified object, which is converted to a string as
     * defined in <i>The Java Language Specification</i>, Section 15.17.1.1.
     */
    public AssertionError(Object detailMessage) {
        this("" +  detailMessage);
    }

    /**
     * Constructs an AssertionError with its detail message derived from
     * the specified boolean, which is converted to a string as
     * defined in <i>The Java Language Specification</i>, Section 15.17.1.1.
     */
    public AssertionError(boolean detailMessage) {
        this("" +  detailMessage);
    }

    /**
     * Constructs an AssertionError with its detail message derived from
     * the specified char, which is converted to a string as
     * defined in <i>The Java Language Specification</i>, Section 15.17.1.1.
     */
    public AssertionError(char detailMessage) {
        this("" +  detailMessage);
    }

    /**
     * Constructs an AssertionError with its detail message derived from
     * the specified int, which is converted to a string as
     * defined in <i>The Java Language Specification</i>, Section 15.17.1.1.
     */
    public AssertionError(int detailMessage) {
        this("" +  detailMessage);
    }

    /**
     * Constructs an AssertionError with its detail message derived from
     * the specified long, which is converted to a string as
     * defined in <i>The Java Language Specification</i>, Section 15.17.1.1.
     */
    public AssertionError(long detailMessage) {
        this("" +  detailMessage);
    }

    /**
     * Constructs an AssertionError with its detail message derived from
     * the specified float, which is converted to a string as
     * defined in <i>The Java Language Specification</i>, Section 15.17.1.1.
     */
    public AssertionError(float detailMessage) {
        this("" +  detailMessage);
    }

    /**
     * Constructs an AssertionError with its detail message derived from
     * the specified double, which is converted to a string as
     * defined in <i>The Java Language Specification</i>, Section 15.17.1.1.
     */
    public AssertionError(double detailMessage) {
        this("" +  detailMessage);
    }
}
