/*
 * @(#)Terminator.java	1.5 00/02/02
 *
 * Copyright 1999, 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package java.lang;

import sun.misc.Signal;
import sun.misc.SignalHandler;


/**
 * Package-private utility class for setting up and tearing down
 * platform-specific support for termination-triggered shutdowns.
 *
 * @author   Mark Reinhold
 * @version  1.5, 00/02/02
 * @since    1.3
 */

class Terminator {

    private static SignalHandler handler = null;

    /* Invocations of setup and teardown are already synchronized
     * on the shutdown lock, so no further synchronization is needed here
     */

    static void setup() {
	if (handler != null) return;
	SignalHandler sh = new SignalHandler() {
	    public void handle(Signal sig) {
		Shutdown.exit(sig.getNumber() + 0200);
	    }
	};
	handler = sh;
	Signal.handle(new Signal("INT"), sh);
	Signal.handle(new Signal("TERM"), sh);
    }

    static void teardown() {
	/* The current sun.misc.Signal class does not support
	 * the cancellation of handlers
	 */
    }

}
