/*
 * @(#)ByteOrder.java	1.6 01/05/03
 *
 * Copyright 2000 by Sun Microsystems, Inc.  All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 */

package java.nio;


/**
 * A typesafe enumeration for byte orders.
 *
 * @author Mark Reinhold
 * @version 1.6, 01/05/03
 * @since 1.4
 */

public final class ByteOrder {

    private String name;

    private ByteOrder(String name) {
	this.name = name;
    }

    /**
     * Constant denoting big-endian byte order.  In this order, the bytes of a
     * multibyte value are ordered from most significant to least significant.
     * </p>
     */
    public static final ByteOrder BIG_ENDIAN
	= new ByteOrder("BIG_ENDIAN");

    /**
     * Constant denoting little-endian byte order.  In this order, the bytes of
     * a multibyte value are ordered from least significant to most
     * significant.  </p>
     */
    public static final ByteOrder LITTLE_ENDIAN
	= new ByteOrder("LITTLE_ENDIAN");

    /**
     * Constructs a string describing this object.
     *
     * <p> This method returns the string <tt>"BIG_ENDIAN"</tt> for {@link
     * #BIG_ENDIAN} and <tt>"LITTLE_ENDIAN"</tt> for {@link #LITTLE_ENDIAN}.
     * </p>
     *
     * @return  The specified string
     */
    public String toString() {
	return name;
    }

}
