/*
 * @(#)Direct-X-Buffer.java	1.27 01/05/03
 *
 * Copyright 2000 by Sun Microsystems, Inc.  All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 */

// -- This file was mechanically generated: Do not edit! -- //

package java.nio;

import sun.misc.Unsafe;
import sun.nio.ch.DirectBuffer;
import sun.nio.ch.FileChannelImpl;


class DirectByteBuffer

    extends MappedByteBuffer



    implements DirectBuffer
{



    // Cached unsafe-access object
    protected static final Unsafe unsafe = Bits.unsafe();

    // Cached unaligned-access capability
    protected static final boolean unaligned = Bits.unaligned();

    // Base access address, used in all indexing calculations
    protected final long base;

    // Native allocation address; may be smaller than the base due to page-size
    // rounding, or the use of a non-zero offset, or both.
    protected long address;

    // True iff this buffer should be freed
    protected boolean allocated;

    // If this buffer is a view of another buffer then we keep a reference to
    // that buffer so that its memory isn't freed before we're done with it
    protected Object viewedBuffer = null;





    DirectByteBuffer(int cap) {			// package-private

	super(-1, 0, cap, cap, false);
	int ps = Bits.pageSize();
	long a = unsafe.allocateMemory(cap + ps);
	address = a;
	base = a + ps - (a % ps);
	allocated = true;



    }

    DirectByteBuffer(int cap,			// package-private
			   long addr, int off,
			   boolean mapped)
    {

	super(-1, 0, cap, cap, mapped);
	address = addr;
	base = address + off;
	allocated = false;



    }



    // Invoked only by JNI: NewDirectByteBuffer(long, int)
    //
    private DirectByteBuffer(long addr, int cap) {
	super(-1, 0, cap, cap, false);
	address = addr;
	base = addr;
	allocated = false;
    }






















    DirectByteBuffer(DirectByteBuffer db,	// package-private
			       int mark, int pos, int lim, int cap,
			       int off)
    {

	super(mark, pos, lim, cap);
	address = db.address();
	base = db.base() + off;
	allocated = false;
	viewedBuffer = db;



    }

    public ByteBuffer slice() {
	int off = (this.position() << 0);
	return new DirectByteBuffer(this,
					      -1,
					      0,
					      this.remaining(),
					      this.remaining(),
					      off);
    }

    public ByteBuffer duplicate() {
	return new DirectByteBuffer(this,
					      this.markValue(),
					      this.position(),
					      this.limit(),
					      this.capacity(),
					      0);
    }

    public ByteBuffer asReadOnlyBuffer() {

	return new DirectByteBufferR(this,
					   this.markValue(),
					   this.position(),
					   this.limit(),
					   this.capacity(),
					   0);



    }



    public long base() {
	return base;
    }

    public long address() {
	return address;
    }

    private long ix(int i) {
	return base + (i << 0);
    }

    public byte get() {
	return (unsafe.getByte(ix(nextGetIndex())));
    }

    public byte get(int i) {
	return (unsafe.getByte(ix(checkIndex(i))));
    }



    public ByteBuffer put(byte x) {

	unsafe.putByte(ix(nextPutIndex()), (x));
	return this;



    }

    public ByteBuffer put(int i, byte x) {

	unsafe.putByte(ix(checkIndex(i)), (x));
	return this;



    }

    public ByteBuffer put(ByteBuffer src) {

	if (src instanceof DirectByteBuffer) {
	    if (src == this)
		throw new IllegalArgumentException();
	    DirectByteBuffer sb = (DirectByteBuffer)src;
	    int n = sb.remaining();
	    if (n > remaining())
		throw new BufferOverflowException();
	    unsafe.copyMemory(sb.ix(sb.position()),
			      ix(position()),
			      n << 0);
	    sb.position(sb.position() + n);
	    position(position() + n);
	} else {
	    super.put(src);
	}
	return this;



    }

    public ByteBuffer compact() {

	unsafe.copyMemory(ix(position()), ix(0),
			  remaining() << 0);
	position(remaining());
	limit(capacity());
	return this;



    }

    public boolean isDirect() {
	return true;
    }

    public boolean isReadOnly() {
	return false;
    }







































    byte _get(int i) {				// package-private
	return unsafe.getByte(base + i);
    }

    void _put(int i, byte b) {			// package-private

	unsafe.putByte(base + i, b);



    }

    protected void finalize() {		
	if (allocated)
	    free();
	else if (isAMappedBuffer) {
	    FileChannelImpl.unmap(this);
	    isAMappedBuffer = false;
	}
    }

    synchronized void free() {				// package-private
	if (allocated) {
	    unsafe.freeMemory(this.address);
	    allocated = false;
	}
    }




    private char getChar(long a) {
	if (unaligned) {
	    char x = unsafe.getChar(a);
	    return (nativeByteOrder ? x : Bits.swap(x));
	}
	return Bits.getChar(a, bigEndian);
    }

    public char getChar() {
	return getChar(ix(nextGetIndex((1 << 1))));
    }

    public char getChar(int i) {
	return getChar(ix(checkIndex(i, (1 << 1))));
    }



    private ByteBuffer putChar(long a, char x) {

	if (unaligned)
	    unsafe.putChar(a, nativeByteOrder ? x : Bits.swap(x));
	else
	    Bits.putChar(a, x, bigEndian);
	return this;



    }

    public ByteBuffer putChar(char x) {

	putChar(ix(nextPutIndex((1 << 1))), x);
	return this;



    }

    public ByteBuffer putChar(int i, char x) {

	putChar(ix(checkIndex(i, (1 << 1))), x);
	return this;



    }

    public CharBuffer asCharBuffer() {
	if (!unaligned && ((base + position()) % (1 << 1) != 0)) {
	    return (bigEndian
		    ? (CharBuffer)(new ByteBufferAsCharBufferB(this))
		    : (CharBuffer)(new ByteBufferAsCharBufferB(this)));
	} else {
	    return (nativeByteOrder
		    ? (CharBuffer)(new DirectCharBufferU(this))
		    : (CharBuffer)(new DirectCharBufferS(this)));
	}
    }




    private short getShort(long a) {
	if (unaligned) {
	    short x = unsafe.getShort(a);
	    return (nativeByteOrder ? x : Bits.swap(x));
	}
	return Bits.getShort(a, bigEndian);
    }

    public short getShort() {
	return getShort(ix(nextGetIndex((1 << 1))));
    }

    public short getShort(int i) {
	return getShort(ix(checkIndex(i, (1 << 1))));
    }



    private ByteBuffer putShort(long a, short x) {

	if (unaligned)
	    unsafe.putShort(a, nativeByteOrder ? x : Bits.swap(x));
	else
	    Bits.putShort(a, x, bigEndian);
	return this;



    }

    public ByteBuffer putShort(short x) {

	putShort(ix(nextPutIndex((1 << 1))), x);
	return this;



    }

    public ByteBuffer putShort(int i, short x) {

	putShort(ix(checkIndex(i, (1 << 1))), x);
	return this;



    }

    public ShortBuffer asShortBuffer() {
	if (!unaligned && ((base + position()) % (1 << 1) != 0)) {
	    return (bigEndian
		    ? (ShortBuffer)(new ByteBufferAsShortBufferB(this))
		    : (ShortBuffer)(new ByteBufferAsShortBufferB(this)));
	} else {
	    return (nativeByteOrder
		    ? (ShortBuffer)(new DirectShortBufferU(this))
		    : (ShortBuffer)(new DirectShortBufferS(this)));
	}
    }




    private int getInt(long a) {
	if (unaligned) {
	    int x = unsafe.getInt(a);
	    return (nativeByteOrder ? x : Bits.swap(x));
	}
	return Bits.getInt(a, bigEndian);
    }

    public int getInt() {
	return getInt(ix(nextGetIndex((1 << 2))));
    }

    public int getInt(int i) {
	return getInt(ix(checkIndex(i, (1 << 2))));
    }



    private ByteBuffer putInt(long a, int x) {

	if (unaligned)
	    unsafe.putInt(a, nativeByteOrder ? x : Bits.swap(x));
	else
	    Bits.putInt(a, x, bigEndian);
	return this;



    }

    public ByteBuffer putInt(int x) {

	putInt(ix(nextPutIndex((1 << 2))), x);
	return this;



    }

    public ByteBuffer putInt(int i, int x) {

	putInt(ix(checkIndex(i, (1 << 2))), x);
	return this;



    }

    public IntBuffer asIntBuffer() {
	if (!unaligned && ((base + position()) % (1 << 2) != 0)) {
	    return (bigEndian
		    ? (IntBuffer)(new ByteBufferAsIntBufferB(this))
		    : (IntBuffer)(new ByteBufferAsIntBufferB(this)));
	} else {
	    return (nativeByteOrder
		    ? (IntBuffer)(new DirectIntBufferU(this))
		    : (IntBuffer)(new DirectIntBufferS(this)));
	}
    }




    private long getLong(long a) {
	if (unaligned) {
	    long x = unsafe.getLong(a);
	    return (nativeByteOrder ? x : Bits.swap(x));
	}
	return Bits.getLong(a, bigEndian);
    }

    public long getLong() {
	return getLong(ix(nextGetIndex((1 << 3))));
    }

    public long getLong(int i) {
	return getLong(ix(checkIndex(i, (1 << 3))));
    }



    private ByteBuffer putLong(long a, long x) {

	if (unaligned)
	    unsafe.putLong(a, nativeByteOrder ? x : Bits.swap(x));
	else
	    Bits.putLong(a, x, bigEndian);
	return this;



    }

    public ByteBuffer putLong(long x) {

	putLong(ix(nextPutIndex((1 << 3))), x);
	return this;



    }

    public ByteBuffer putLong(int i, long x) {

	putLong(ix(checkIndex(i, (1 << 3))), x);
	return this;



    }

    public LongBuffer asLongBuffer() {
	if (!unaligned && ((base + position()) % (1 << 3) != 0)) {
	    return (bigEndian
		    ? (LongBuffer)(new ByteBufferAsLongBufferB(this))
		    : (LongBuffer)(new ByteBufferAsLongBufferB(this)));
	} else {
	    return (nativeByteOrder
		    ? (LongBuffer)(new DirectLongBufferU(this))
		    : (LongBuffer)(new DirectLongBufferS(this)));
	}
    }




    private float getFloat(long a) {
	if (unaligned) {
	    float x = unsafe.getFloat(a);
	    return (nativeByteOrder ? x : Bits.swap(x));
	}
	return Bits.getFloat(a, bigEndian);
    }

    public float getFloat() {
	return getFloat(ix(nextGetIndex((1 << 2))));
    }

    public float getFloat(int i) {
	return getFloat(ix(checkIndex(i, (1 << 2))));
    }



    private ByteBuffer putFloat(long a, float x) {

	if (unaligned)
	    unsafe.putFloat(a, nativeByteOrder ? x : Bits.swap(x));
	else
	    Bits.putFloat(a, x, bigEndian);
	return this;



    }

    public ByteBuffer putFloat(float x) {

	putFloat(ix(nextPutIndex((1 << 2))), x);
	return this;



    }

    public ByteBuffer putFloat(int i, float x) {

	putFloat(ix(checkIndex(i, (1 << 2))), x);
	return this;



    }

    public FloatBuffer asFloatBuffer() {
	if (!unaligned && ((base + position()) % (1 << 2) != 0)) {
	    return (bigEndian
		    ? (FloatBuffer)(new ByteBufferAsFloatBufferB(this))
		    : (FloatBuffer)(new ByteBufferAsFloatBufferB(this)));
	} else {
	    return (nativeByteOrder
		    ? (FloatBuffer)(new DirectFloatBufferU(this))
		    : (FloatBuffer)(new DirectFloatBufferS(this)));
	}
    }




    private double getDouble(long a) {
	if (unaligned) {
	    double x = unsafe.getDouble(a);
	    return (nativeByteOrder ? x : Bits.swap(x));
	}
	return Bits.getDouble(a, bigEndian);
    }

    public double getDouble() {
	return getDouble(ix(nextGetIndex((1 << 3))));
    }

    public double getDouble(int i) {
	return getDouble(ix(checkIndex(i, (1 << 3))));
    }



    private ByteBuffer putDouble(long a, double x) {

	if (unaligned)
	    unsafe.putDouble(a, nativeByteOrder ? x : Bits.swap(x));
	else
	    Bits.putDouble(a, x, bigEndian);
	return this;



    }

    public ByteBuffer putDouble(double x) {

	putDouble(ix(nextPutIndex((1 << 3))), x);
	return this;



    }

    public ByteBuffer putDouble(int i, double x) {

	putDouble(ix(checkIndex(i, (1 << 3))), x);
	return this;



    }

    public DoubleBuffer asDoubleBuffer() {
	if (!unaligned && ((base + position()) % (1 << 3) != 0)) {
	    return (bigEndian
		    ? (DoubleBuffer)(new ByteBufferAsDoubleBufferB(this))
		    : (DoubleBuffer)(new ByteBufferAsDoubleBufferB(this)));
	} else {
	    return (nativeByteOrder
		    ? (DoubleBuffer)(new DirectDoubleBufferU(this))
		    : (DoubleBuffer)(new DirectDoubleBufferS(this)));
	}
    }

}
