/*
 * @(#)StringCharBuffer.java	1.9 01/05/03
 *
 * Copyright 2000 by Sun Microsystems, Inc.  All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 */

package java.nio;


// ## Use reflection to share the string's character array

class StringCharBuffer					// package-private
    extends CharBuffer
{
    String str;

    StringCharBuffer(String s, int start, int end) {	// package-private
	super(-1, 0, end - start, end - start);
	int n = s.length();
	if ((start < 0) || (start > n) || (end < start) || (end > n))
	    throw new IndexOutOfBoundsException();
	str = (((start == 0) && (end == s.length()))
	       ? s : s.substring(start, end));
    }

    public CharBuffer slice() {
	return new StringCharBuffer(str, position(), limit());
    }

    private StringCharBuffer(String s, int mark, 	// package-private
			     int pos, int limit, int cap)
    {
	super(mark, pos, limit, cap);
	str = s;
    }

    public CharBuffer duplicate() {
	return new StringCharBuffer(str, markValue(),
				    position(), limit(), capacity());
    }

    public CharBuffer asReadOnlyBuffer() {
	return duplicate();
    }

    public final char get() {
	return str.charAt(nextGetIndex());
    }

    public final char get(int index) {
	return str.charAt(index);
    }

    // ## Override bulk get methods for better performance

    public final CharBuffer put(char c) {
	throw new ReadOnlyBufferException();
    }

    public final CharBuffer put(int index, char c) {
	throw new ReadOnlyBufferException();
    }

    public final CharBuffer compact() {
	throw new ReadOnlyBufferException();
    }

    public final boolean isReadOnly() {
	return true;
    }

    final String toString(int start, int end) {
	return str.substring(start, end);
    }

    public final CharSequence subSequence(int start, int end) {
	return new StringCharBuffer(str, start, end);
    }

    public boolean isDirect() {
	return false;
    }

}
