/* 
 * @(#)KerberosPrincipal.java	1.4 00/11/28   
 *   
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.    
 *     
 * This software is the proprietary information of Sun Microsystems, Inc.    
 * Use is subject to license terms.   
 *   
 */  
  
package javax.security.auth.kerberos;

import sun.security.krb5.PrincipalName;
import sun.security.krb5.KrbException;

/**
 * This class encapsulates a Kerberos principal.
 *
 * @author Mayank Upadhyay
 * @version 1.4, 11/28/00
 * @since 1.4
 */

public class KerberosPrincipal 
    implements java.security.Principal, java.io.Serializable {

    private String fullName;
    private String realm;

    /** 
     * Constructs a KerberosPrincipal by parsing the string that is
     * provided. This string is assumed to contain a name in the format
     * that is specified in RFC 1964 as "Kerberos Principal Name Form"
     * 
     * @param strName the principal name
     * @throws IllegalArgumentException if strName is improperly
     * formatted.
     */
    /*
     * Note: Declare IllegalArgumentException just to document it.
     * TBD:  Research what encoding would be most appropriate to use
     *       when converting the String to bytes. And document that.
     */
    public KerberosPrincipal(String strName) 
	throws IllegalArgumentException {

	PrincipalName krb5Principal = null;

	try {
	    // Appends the default realm if it is missing
	    krb5Principal  = new PrincipalName(strName);
	} catch (KrbException e) {
	    throw new IllegalArgumentException(e.getMessage());
	}
	 
	fullName = krb5Principal.toString();
	realm = krb5Principal.getRealmString();
    }

    /**
     * Returns the realm component of this Kerberos principal.
     */
    public final String getRealmAsString() {
	return realm;
    }

    /**
     * Returns a hashcode for this principal.
     */
    public int hashCode() {
	return getName().hashCode();
    }

    /**
     * Compares this principal to the another Object.
     *
     * @param other the Object to compare to
     * @return true if the Object passed in represents the same principal
     * as this one, false otherwise.
     */
    public boolean equals(Object other) {

	if (other == this)
	    return true;

	if (other instanceof KerberosPrincipal) {
	    String myFullName = getName();
	    String otherFullName = ((KerberosPrincipal) other).getName();
	    return myFullName.equals(otherFullName);
	} else
	    return false;
    }

    /**
     * Returns the principal name with the realm appended to it.
     */
    public String getName() {
	return fullName;
    }
    
    // Inherits javadocs from Object
    public String toString() {
	return getName();
    }
}
