/*
 * @(#)BasicDragGestureRecognizer.java	1.1 00/11/16
 *
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
package javax.swing.plaf.basic;

import java.awt.event.*;
import javax.swing.*;

/**
 * Default drag gesture recognition for drag operations performed by classses
 * that have a <code>dragEnabled</code> property.  The gesture for a drag in
 * this package is a mouse press over a selection followed by some movement
 * by enough pixels to keep it from being treated as a click.
 *
 * @author  Timothy Prinzing
 * @version 1.1 11/16/00
 */
class BasicDragGestureRecognizer extends MouseAdapter implements MouseMotionListener {

    /**
     * Invoked when a mouse button has been pressed on a component.
     */
    public void mousePressed(MouseEvent e) {
	if (isDragPossible(e)) {
	    dndArmed = e;
	    e.consume();
	}
    }

    public void mouseDragged(MouseEvent e) {
	if (dndArmed != null) {
	    e.consume();
	    int dx = Math.abs(e.getX() - dndArmed.getX());
	    int dy = Math.abs(e.getY() - dndArmed.getY());
	    int npixels = getSensitivity();
	    if ((dx > npixels) || (dy > npixels)) {
		// start transfer... shouldn't be a click at this point
		//didDrag = true;
		JComponent c = getComponent(e);
		TransferHandler th = c.getTransferHandler();
		th.exportAsDrag(c, e, TransferHandler.COPY);
		dndArmed = null;
	    }
	}
    }

    /**
     * Invoked when the mouse button has been moved on a component
     * (with no buttons no down).
     */
    public void mouseMoved(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
	dndArmed = null;
    }

    /**
     * Determines if the following are true:
     * <ul>
     * <li>the press event is located over a selection
     * <li>the dragEnabled property is true
     * <li>A TranferHandler is installed
     * </ul>
     * <p>
     * This is implemented to check for a TransferHandler.
     * Subclasses should perform the remaining conditions.
     */
    protected boolean isDragPossible(MouseEvent e) {
	JComponent c = getComponent(e);
	if (c != null) {
	    TransferHandler th = c.getTransferHandler();
	    return (th != null);
	}
	return false;
    }

    protected JComponent getComponent(MouseEvent e) {
	Object src = e.getSource();
	if (src instanceof JComponent) {
	    JComponent c = (JComponent) src;
	    return c;
	}
	return null;
    }

    /**
     * The number of pixels to allow before starting a drag.
     */
    int getSensitivity() {
	return 3;
    }

    private MouseEvent dndArmed;

}
