/*
 * @(#)DateFormatter.java	1.4 01/03/05
 *
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */
package javax.swing.text;

import java.awt.event.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import javax.swing.text.*;

/**
 * DateFormatter is an <code>InternationalFormatter</code> that does its
 * formatting by way of an instance of <code>java.text.DateFormat</code>.
 * <p>
 * <strong>Warning:</strong>
 * Serialized objects of this class will not be compatible with
 * future Swing releases. The current serialization support is
 * appropriate for short term storage or RMI between applications running
 * the same version of Swing.  As of 1.4, support for long term storage
 * of all JavaBeans<sup><font size="-2">TM</font></sup>
 * has been added to the <code>java.beans</code> package.
 * Please see {@link java.beans.XMLEncoder}.
 *
 * @see java.text.DateFormat
 *
 * @version 1.4 03/05/01
 * @since 1.4
 */
public class DateFormatter extends InternationalFormatter {
    /**
     * This is shorthand for
     * <code>new DateFormatter(DateFormat.getDateInstance())</code>.
     */
    public DateFormatter() {
        this(DateFormat.getDateInstance());
    }

    /**
     * Returns a DateFormatter configured with the specified
     * <code>Format</code> instance.
     *
     * @param format Format used to dictate legal values
     */
    public DateFormatter(DateFormat format) {
        super(format);
        setFormat(format);
    }

    /**
     * Sets the format that dictates the legal values that can be edited
     * and displayed.
     * <p>
     * If you have used the nullary constructor the value of this property
     * will be determined for the current locale by way of the
     * <code>Dateformat.getDateInstance()</code> method.
     *
     * @param format DateFormat instance used for converting from/to Strings
     */
    public void setFormat(DateFormat format) {
        super.setFormat(format);
    }

    /**
     * Creates the <code>Filter</code> instance that is used to enforce a
     * policy in the <code>JFormattedTextField</code>.
     */
    DefaultFormatter.Filter createFilter() {
        return new Filter();
    }

    /**
     * Returns the Calendar that <code>DateFormat</code> is associated with,
     * or if the <code>Format</code> is not a <code>DateFormat</code>
     * <code>Calendar.getInstance</code> is returned.
     */
    private Calendar getCalendar() {
        Format f = getFormat();

        if (f instanceof DateFormat) {
            return ((DateFormat)f).getCalendar();
        }
        return Calendar.getInstance();
    }


    /**
     * Filter extends <code>InternationalTextFormatter.Filter</code> adding
     * behavior for incrementing/decrementing the date.
     */
    class Filter extends InternationalFormatter.Filter {
        /**
         * Returns true, as DateFormatterFilter will support
         * incrementing/decrementing of the value.
         */
        boolean getSupportsIncrement() {
            return true;
        }

        /**
         * Adjusts the Date if FieldPosition identifies a known calendar
         * field.
         */
        Object adjustValue(Object value, Map attributes, int direction) throws
                      BadLocationException, ParseException {
            if (attributes != null && (value instanceof Date)) {
                Iterator attrs = attributes.keySet().iterator();

                while (attrs.hasNext()) {
                    Object key = attrs.next();
                    int field;

                    if (key instanceof DateFormat.Field && (field = 
                           ((DateFormat.Field)key).getCalendarField()) != -1) {
                        Calendar calendar = getCalendar();

                        if (calendar != null) {
                            calendar.setTime((Date)value);

                            int fieldValue = calendar.get(field);

                            try {
                                calendar.add(field, direction);
                                value = calendar.getTime();
                            } catch (Throwable th) {
                                value = null;
                            }
                            return value;
                        }
                    }
                }
            }
            return null;
        }
    }
}
